'use strict';

exports = module.exports = {
    getRemoteSupport,
    enableRemoteSupport,

    getConfig,

    _sshInfo: sshInfo
};

const assert = require('assert'),
    BoxError = require('./boxerror.js'),
    constants = require('./constants.js'),
    eventlog = require('./eventlog.js'),
    path = require('path'),
    paths = require('./paths.js'),
    safe = require('safetydance'),
    settings = require('./settings.js'),
    shell = require('./shell.js');

// the logic here is also used in the cloudron-support tool
const AUTHORIZED_KEYS_CMD = path.join(__dirname, 'scripts/remotesupport.sh');

function sshInfo() {
    let filePath, user;

    if (constants.TEST) {
        filePath = path.join(paths.baseDir(), 'authorized_keys');
        user = process.getuid();
    } else {
        filePath = `/home/${constants.SUPPORT_USERNAME}/.ssh/authorized_keys`;
        user = constants.SUPPORT_USERNAME;
    }

    return { filePath, user };
}

async function getRemoteSupport() {
    const [error, stdoutResult] = await safe(shell.promises.sudo('support', [ AUTHORIZED_KEYS_CMD, 'is-enabled', sshInfo().filePath ], {}));
    if (error) throw new BoxError(BoxError.FS_ERROR, error);

    return stdoutResult.trim() === 'true';
}

async function enableRemoteSupport(enable, auditSource) {
    assert.strictEqual(typeof enable, 'boolean');
    assert.strictEqual(typeof auditSource, 'object');

    const si = sshInfo();
    const [error] = await safe(shell.promises.sudo('support', [ AUTHORIZED_KEYS_CMD, enable ? 'enable' : 'disable', si.filePath, si.user ], {}));
    if (error) throw new BoxError(BoxError.FS_ERROR, error);

    await eventlog.add(eventlog.ACTION_SUPPORT_SSH, auditSource, { enable });
}

async function getConfig() {
    const value = await settings.getJson(settings.SUPPORT_CONFIG_KEY);
    return value || {
        email: 'support@cloudron.io',
        remoteSupport: true,
        ticketFormBody:
            'Use this form to open support tickets. You can also write directly to [support@cloudron.io](mailto:support@cloudron.io).\n\n'
            + '* [Knowledge Base & App Docs](https://docs.cloudron.io/apps/?support_view)\n'
            + '* [Custom App Packaging & API](https://docs.cloudron.io/custom-apps/tutorial/?support_view)\n'
            + '* [Forum](https://forum.cloudron.io/)\n\n',
        submitTickets: true
    };
}
